'''Game main module.

Contains the entry point used by the run_game.py script.

Feel free to put all your game code here, or in other modules in this "lib"
directory.
'''
from gameobjects import *
from vector import *

import sys
import traceback
from OpenGL.GL import *
from OpenGL.GLU import *
from OpenGL.GLUT import *
import pygame, pygame.image
from pygame.locals import *

# Some global variables, because they are simply awsome
WIDTH = 640
HEIGHT = 480
GAMESTATE = 1
ViewMode = 1
ShowHint = False
CurLevel = 1
NumLevel = 3

def loadtexture(path):
	textureSurface = pygame.image.load(path)
	textureData = pygame.image.tostring(textureSurface, "RGBX", 1)
	texture = glGenTextures(1)
	glBindTexture(GL_TEXTURE_2D, texture)
	glTexImage2D( GL_TEXTURE_2D, 0, GL_RGBA, textureSurface.get_width(), textureSurface.get_height(), 0,
				  GL_RGBA, GL_UNSIGNED_BYTE, textureData );
	glTexParameterf(GL_TEXTURE_2D, GL_TEXTURE_MAG_FILTER, GL_NEAREST)
	glTexParameterf(GL_TEXTURE_2D, GL_TEXTURE_MIN_FILTER, GL_NEAREST)
	return texture

    
def loadtextures():
	global textures
	textures = {}
	textures["Start"]    = loadtexture("data/img/start.bmp")
	textures["Instruct"] = loadtexture("data/img/instruct.bmp")
	textures["End"]      = loadtexture("data/img/end.bmp")

def loadlevel(path):
	f = open(path, "r")
	data = f.readlines()
	f.close()
	cleandata = []
	for n in data:
		if n[0] == "#":
			continue
		elif n == "\n":
			continue
		else:
			cleandata.append(n.rstrip())

	if cleandata[0] == "Sphere":
		gameobject = Sphere(int(cleandata[1]), int(cleandata[2]), int(cleandata[3]))
	for n in cleandata[4:]:
		m = n.split()
		gameobject.segments[int(m[0])].quads[int(m[1])].color = Vec3(float(m[2]), float(m[3]), float(m[4]))
		if int(m[0]) not in gameobject.relevant:
			gameobject.relevant.append(int(m[0]))
	gameobject.relevant.sort()
	for n in gameobject.segments:
			n.makeDL()
	return gameobject

def gameInit():
	#loadtextures()
	global yrot, zrot, gameobject, m_pos, CurLevel
	yrot = 90
	zrot = 40
	m1_pressed = False
	m_pos = pygame.mouse.get_rel()

	gameobject = loadlevel("data/level_%d.txt" % CurLevel)
	gameobject.scramble()
	
def handlekbdinput(event):
	global Running, GAMESTATE, yrot, zrot, gameobject, ViewMode, ShowHint
	if GAMESTATE == 1:
		GAMESTATE = 2
		return
	elif GAMESTATE == 3:
		GAMESTATE = 4
		return
	elif GAMESTATE == 5:
		Running = False
		return
	elif GAMESTATE == 2:
		if event.key == K_ESCAPE:
			print "Quitter!"
			sys.exit(0)
		elif event.key == K_LEFT:	gameobject.rotate(gameobject.actvseg,-1)
		elif event.key == K_RIGHT:	gameobject.rotate(gameobject.actvseg, 1)
		elif event.key == K_UP:		gameobject.prev()
		elif event.key == K_DOWN:	gameobject.next()
		elif event.key == K_h: ShowHint = not ShowHint
		elif event.key == K_w:
			if ViewMode == 1:
				glPolygonMode(GL_FRONT_AND_BACK, GL_LINE);
				ViewMode = 2
			elif ViewMode == 2:
				glPolygonMode(GL_FRONT_AND_BACK, GL_POINT);
				ViewMode = 3
			elif ViewMode == 3:
				glPolygonMode(GL_FRONT_AND_BACK, GL_FILL);
				ViewMode = 1
	
def mousemove(event):
	global m_pos, yrot, zrot
	relx, rely = event.rel
	m1, m2, m3 = event.buttons
	if m1:
		yrot += relx*0.5
		zrot += rely*0.5
		m_pos = event.pos

def drawSplash():
	setOrtho()
	
	putString(235, 160, 0, "Twistball! Wohooo!")
	putString(120, 190, 0, "Just rotate each segment until it makes sense")
	putString(250, 210, 0, "(check the hints)")
	putString(130, 250, 0, "Controls:")
	putString(130, 270, 0, "Up/down arrows - Select a different segment")
	putString(130, 290, 0, "Left/right - Rotate the selected segment")
	putString(130, 310, 0, "Mouse - Click and drag to rotate the view")
	putString(130, 330, 0, "W - switch between view modes (Useless ;p)")	
	putString(130, 350, 0, "H - Show hint (and fps..)")
	
	resetProjection()

	
def drawGame():
	global FPS
	glClear(GL_COLOR_BUFFER_BIT|GL_DEPTH_BUFFER_BIT)
	glLoadIdentity()

	glTranslatef(0, 0, -25)
	
	glRotatef(yrot, 0, 1, 0)
	glRotatef(zrot, 0, 0, 1)
	
	gameobject.draw()
	
	if not ShowHint:
		return
	
	# Draw ze hint
	setOrtho()
	x = 0
	y = 0
	glLoadIdentity()
	glBegin(GL_QUADS)
	for i in range(gameobject.relevant[0], gameobject.relevant[-1]+1):
		x = 0
		for m in gameobject.segments[i].quads:
			glColor3f(m.color.x, m.color.y, m.color.z)
			glVertex2f(x*16,    y*16)
			glVertex2f((x*16)+16, y*16)
			glVertex2f((x*16)+16, (y*16)+16)
			glVertex2f(x*16,    (y*16)+16)
			x += 1
		y += 1
	glEnd()
	
	glColor4f(1,1,1,1)
	putString(5, 475, 0, "FPS: %d" % FPS)
	resetProjection()
	
def drawLevelWin():
	global CurLevel
	
	drawGame()
	setOrtho()
	
	glLoadIdentity()
	glColor4f(0,0,0,0.4)
	glBegin(GL_QUADS)
	glVertex2f( 0, 480)
	glVertex2f( 0, 0)
	glVertex2f( 640, 0)
	glVertex2f( 640, 480)
	glEnd()
	
	glColor4f(1,1,1,1)
	putString(200, 240, 0, "Grats, you completed level %d!" % (CurLevel-1))
	resetProjection()

def drawEnd():
	drawGame()
	glLoadIdentity()
	setOrtho()
	glColor4f(0,0,0,0.4)
	glBegin(GL_QUADS)
	glVertex2f( 0, 480)
	glVertex2f( 0, 0)
	glVertex2f( 640, 0)
	glVertex2f( 640, 480)
	glEnd()
	
	glColor4f(1,1,1,1)
	
	putString(270, 220, 0, "You won!")
	putString(225, 260, 0, "Press any key to exit")
	resetProjection()


def putString(x, y, z, string):
	glRasterPos3f(x, y, z)
	for c in string:
		glutBitmapCharacter(GLUT_BITMAP_HELVETICA_18, ord(c));

def draw():
	if GAMESTATE == 1:
		drawSplash()
		return
	elif GAMESTATE == 2:
		drawGame()
		return
	elif GAMESTATE == 3:
		drawLevelWin()
		return
	elif GAMESTATE == 5:
		drawEnd()
		return



def main():
	global Running, GAMESTATE, FPS, CurLevel, NumLevel, gameobject
	Running = True
	FPS = 0
	
	video_flags = OPENGL|DOUBLEBUF
	
	pygame.init()
	pygame.display.set_mode((WIDTH, HEIGHT), video_flags)

	resize((WIDTH, HEIGHT))
	screenInit(WIDTH, HEIGHT)
	
	glutInit(sys.argv) # so we can draw text..
	
	gameInit()
	
	
	
	lasttick = pygame.time.get_ticks()
	frames   = 0
	while Running:
		event = pygame.event.poll()
		if event.type == QUIT:
			Running = False
		elif event.type == KEYDOWN:
			handlekbdinput(event)
		elif event.type == MOUSEMOTION:
			mousemove(event)
		
		# Fps..
		frames += 1
		ticksnow = pygame.time.get_ticks()
		diff = ticksnow - lasttick
		if diff > 1000:
			FPS = frames / (diff / 1000)
			lasttick = ticksnow
			frames = 0
					
		# check if we have a winner..
		tmp = gameobject.segments[gameobject.relevant[0]].rot
		win = 0
		for i in gameobject.relevant:
			if gameobject.segments[i].rot == tmp:
				win += 1
		if win == len(gameobject.relevant) and GAMESTATE == 2:
			CurLevel += 1
			GAMESTATE = 3
			if CurLevel > NumLevel:
				GAMESTATE = 5
		if GAMESTATE == 4:
			gameobject = loadlevel("data/level_%d.txt" % CurLevel)
			gameobject.scramble()
			GAMESTATE = 2

		draw()
			
		pygame.display.flip()


def resize((width, height)):
	if height==0:
		height=1
	glViewport(0, 0, width, height)
	glMatrixMode(GL_PROJECTION)
	glLoadIdentity()
	gluPerspective(45, 1.0*width/height, 0.1, 100.0)
	glMatrixMode(GL_MODELVIEW)
	glLoadIdentity()
    
def screenInit(width, height):
	glShadeModel(GL_SMOOTH)
	glClearColor(0.0, 0.0, 0.0, 1.0)
	glBlendFunc(GL_SRC_ALPHA,GL_ONE_MINUS_SRC_ALPHA);
	glEnable(GL_BLEND);
	glEnable(GL_DEPTH_TEST)
	glDepthFunc(GL_LEQUAL)
	glHint(GL_PERSPECTIVE_CORRECTION_HINT, GL_NICEST)
    
def setOrtho():
	# switch to projection mode
	glMatrixMode(GL_PROJECTION)
	glPushMatrix()
	glLoadIdentity()
	# set a 2D orthographic projection
	gluOrtho2D(0, WIDTH, 0, HEIGHT)
	# invert the y axis, down is positive
	glScalef(1, -1, 1)
	# mover the origin from the bottom left corner
	# to the upper left corner
	glTranslatef(0, -HEIGHT, 0)
	glMatrixMode(GL_MODELVIEW)
	
def resetProjection():
	glMatrixMode(GL_PROJECTION)
	glPopMatrix()
	glMatrixMode(GL_MODELVIEW)
